# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements a class responsible for manager all triggers available.
"""

import os
import imp
from TextFlow.core.constants import TRIGGER_DIR

class TriggerManager(object):
    """
    This class is responsible for loading of all triggers available.
    """
    
    def __init__(self, document_manager):
        """
        Constructor.
        
        @param document_manager: Reference to the DocumentManager.
        @type document_manager: A DocumentManager object.
        """
        self.document_manager = document_manager
        self.triggers = []
        self.triggers_class = []
        self.shortcuts = {}
        self.sticky_shortcuts = {}
        self.sticky_keys = {}
        self.last_shortcut = None
        
        self.__get_triggers()
        self.__load_triggers()
        
    #################### Private Methods ####################
    
    def __get_triggers(self):
        """
        Get a list with all files that implement triggers.
        """
        files_dir = os.listdir(TRIGGER_DIR)
        files = list(files_dir)
        
        for f in files:
            f_name = f.split('.')
            try:
                if f_name[1] != 'py':
                    continue
                else:
                    self.triggers.append(f_name[0])
            except IndexError:
                pass
        
        self.triggers.remove('__init__')
        
    def __load_triggers(self):
        """
        Load all triggers available.
        """
        for p in self.triggers:
            #loading class...
            module_path = TRIGGER_DIR + '/' + p + '.py'
            module = imp.load_source(p, module_path)
            trigger_class = getattr(module, p)
            
            shortcut = module.shortcut
            
            trigger_obj = trigger_class(self.document_manager)

            self.shortcuts[shortcut] = trigger_obj.activate
            
            if module.sticky:
                self.sticky_shortcuts[shortcut] = trigger_obj.sticky_release
                self.sticky_keys[shortcut] = module.sticky_keys
                
