# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements the trigger of "ctrl+tab".
"""

import re

shortcut = "ctrl+" +  u' '
sticky = False

class SnippetActivate(object):
    
    """
    This class implements the snippet activate operation.
    """
    
    def __init__(self, document_manager):
        """
        Constructor.
        
        @param document_manager: Reference to the DocumentManager.
        @type document_manager: A DocumentManager object.
        """
        
        self.document_manager = document_manager
    
    def activate(self):
        """
        Activate the snippet.
        """
        view = self.document_manager.get_active_view()
        buffer = view.buffer
        snippets = view.snippets
        
        insert_mark = buffer.get_insert()
        insert_iter = buffer.get_iter_at_mark(insert_mark)
        start_iter = insert_iter.copy()
        
        self.__find_word_start(start_iter)
        key = buffer.get_text(start_iter, insert_iter)
        buffer.delete(start_iter, insert_iter)
        try:
            snippets.active(key)
        except KeyError:
            buffer.insert_at_cursor(key)
            
    def pos_activate(self):
        pass
    
    def __find_word_start(self, iterator):
        pattern = re.compile("[a-z|0-9]")
        symbols = ('!', '@', '#', '$', '%', '&', '*',
                   '(', ')', '-' , '+', '.', ',', '~', '^')
        iterator.backward_char()
        if iterator.get_char() in symbols:
            return
        while True:
            char = iterator.get_char()
            if not(re.match(pattern, char)):
                iterator.forward_char()
                return
            elif iterator.starts_line():
                return
            else:
                iterator.backward_char()

