# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements the trigger of "ctrl+u".
"""

shortcut = 'ctrl+ '
sticky = False

class WordComplete(object):
    
    def __init__(self, document_manager):
        """
        Constructor.
        
        @param document_manager: Reference to the DocumentManager.
        @type document_manager: A DocumentManager object.
        """
        
        self.document_manager = document_manager
    
    def activate(self):
        """
        Snippet active.
        """

        view = self.document_manager.get_active_view()
        doc = self.document_manager.get_active_view().buffer
        
        insert_mark = doc.get_insert()
        insert_iter = doc.get_iter_at_mark(insert_mark)
        start_iter = insert_iter.copy()

        self.__find_word_start(start_iter)
        key = doc.get_text(start_iter, insert_iter)
        
        if key != "":
            if view.word_complete.pw == None:
                view.word_complete.complete(key)
            return True
        else:
            return False
            
    def __find_word_start(self, iterator):
        if iterator.ends_word() or iterator.inside_word():
            iterator.backward_word_start()
        
        return
        
        

